@@ -4,7 +4,7 @@
  *
  * Copyright (C) 2005 The GNOME Foundation
  * Copyright (C) 2006, 2007, 2008 Frederic Ruaudel and others (see AUTHORS)
- * Copyright (C) 2009 Pierre Wieser and others (see AUTHORS)
+ * Copyright (C) 2009, 2010 Pierre Wieser and others (see AUTHORS)
  *
  * This Program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -36,19 +36,19 @@
 #include <gtk/gtk.h>
 #include <string.h>
 
-#include <runtime/na-object-api.h>
-#include <runtime/na-pivot.h>
+#include <api/na-core-utils.h>
+#include <api/na-object-api.h>
 
-#include <common/na-iprefs.h>
-#include <common/na-utils.h>
-#include <common/na-xml-names.h>
-#include <common/na-xml-writer.h>
+#include <core/na-iprefs.h>
+#include <core/na-exporter.h>
 
 #include "base-iprefs.h"
 #include "nact-application.h"
+#include "nact-iprefs.h"
 #include "nact-main-window.h"
 #include "nact-assistant-export.h"
-#include "nact-assistant-export-ask.h"
+#include "nact-export-ask.h"
+#include "nact-export-format.h"
 #include "nact-iactions-list.h"
 
 /* Export Assistant
@@ -87,15 +87,13 @@ struct NactAssistantExportPrivate {
 };
 
 typedef struct {
-	NAObjectAction *action;
-	GSList         *msg;
-	gchar          *fname;
-	gint            format;
+	NAObjectItem *item;
+	GSList       *msg;
+	gchar        *fname;
+	GQuark        format;
 }
 	ExportStruct;
 
-#define IPREFS_EXPORT_ACTIONS_FOLDER_URI		"export-folder-uri"
-
 static BaseAssistantClass *st_parent_class = NULL;
 
 static GType           register_type( void );
@@ -107,12 +105,13 @@ static void            instance_finalize( GObject *application );
 
 static NactAssistantExport *assist_new( BaseWindow *parent );
 
-static gchar          *window_get_iprefs_window_id( BaseWindow *window );
-static gchar          *window_get_toplevel_name( BaseWindow *dialog );
-static gchar          *window_get_ui_filename( BaseWindow *dialog );
+static gchar          *window_get_iprefs_window_id( const BaseWindow *window );
+static gchar          *window_get_toplevel_name( const BaseWindow *dialog );
+static gchar          *window_get_ui_filename( const BaseWindow *dialog );
 
 static void            on_initial_load_dialog( NactAssistantExport *dialog, gpointer user_data );
 static void            on_runtime_init_dialog( NactAssistantExport *dialog, gpointer user_data );
+static void            on_all_widgets_showed( NactAssistantExport *dialog );
 
 static void            assist_initial_load_intro( NactAssistantExport *window, GtkAssistant *assistant );
 static void            assist_runtime_init_intro( NactAssistantExport *window, GtkAssistant *assistant );
@@ -129,11 +128,7 @@ static void            on_folder_selection_changed( GtkFileChooser *chooser, gpo
 
 static void            assist_initial_load_format( NactAssistantExport *window, GtkAssistant *assistant );
 static void            assist_runtime_init_format( NactAssistantExport *window, GtkAssistant *assistant );
-static GtkWidget      *get_gconfschemav1_button( NactAssistantExport *window );
-static GtkWidget      *get_gconfschemav2_button( NactAssistantExport *window );
-static GtkWidget      *get_gconfdump_button( NactAssistantExport *window );
-static GtkWidget      *get_ask_button( NactAssistantExport *window );
-static gint            get_export_format( NactAssistantExport *window );
+static NAExportFormat *get_export_format( NactAssistantExport *window );
 
 static void            assist_initial_load_confirm( NactAssistantExport *window, GtkAssistant *assistant );
 static void            assist_runtime_init_confirm( NactAssistantExport *window, GtkAssistant *assistant );
@@ -227,7 +222,6 @@ iactions_list_iface_init( NactIActionsListInterface *iface )
 
 	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
 
-	iface->selection_changed = on_iactions_list_selection_changed;
 	iface->get_treeview_name = on_iactions_list_get_treeview_name;
 }
 
@@ -237,7 +231,8 @@ instance_init( GTypeInstance *instance, gpointer klass )
 	static const gchar *thisfn = "nact_assistant_export_instance_init";
 	NactAssistantExport *self;
 
-	g_debug( "%s: instance=%p, klass=%p", thisfn, ( void * ) instance, ( void * ) klass );
+	g_debug( "%s: instance=%p (%s), klass=%p",
+			thisfn, ( void * ) instance, G_OBJECT_TYPE_NAME( instance ), ( void * ) klass );
 	g_assert( NACT_IS_ASSISTANT_EXPORT( instance ));
 	self = NACT_ASSISTANT_EXPORT( instance );
 
@@ -256,6 +251,12 @@ instance_init( GTypeInstance *instance, gpointer klass )
 			G_OBJECT( instance ),
 			BASE_WINDOW_SIGNAL_RUNTIME_INIT,
 			G_CALLBACK( on_runtime_init_dialog ));
+
+	base_window_signal_connect(
+			BASE_WINDOW( instance ),
+			G_OBJECT( instance ),
+					BASE_WINDOW_SIGNAL_ALL_WIDGETS_SHOWED,
+			G_CALLBACK( on_all_widgets_showed ));
 }
 
 static void
@@ -264,7 +265,7 @@ instance_dispose( GObject *window )
 	static const gchar *thisfn = "nact_assistant_export_instance_dispose";
 	NactAssistantExport *self;
 
-	g_debug( "%s: window=%p", thisfn, ( void * ) window );
+	g_debug( "%s: window=%p (%s)", thisfn, ( void * ) window, G_OBJECT_TYPE_NAME( window ));
 	g_return_if_fail( NACT_IS_ASSISTANT_EXPORT( window ));
 	self = NACT_ASSISTANT_EXPORT( window );
 
@@ -272,6 +273,8 @@ instance_dispose( GObject *window )
 
 		self->private->dispose_has_run = TRUE;
 
+		nact_iactions_list_dispose( NACT_IACTIONS_LIST( window ));
+
 		/* chain up to the parent class */
 		if( G_OBJECT_CLASS( st_parent_class )->dispose ){
 			G_OBJECT_CLASS( st_parent_class )->dispose( window );
@@ -322,19 +325,19 @@ nact_assistant_export_run( BaseWindow *main_window )
 }
 
 static gchar *
-window_get_iprefs_window_id( BaseWindow *window )
+window_get_iprefs_window_id( const BaseWindow *window )
 {
 	return( g_strdup( "export-assistant" ));
 }
 
 static gchar *
-window_get_toplevel_name( BaseWindow *dialog )
+window_get_toplevel_name( const BaseWindow *dialog )
 {
 	return( g_strdup( "ExportAssistant" ));
 }
 
 static gchar *
-window_get_ui_filename( BaseWindow *dialog )
+window_get_ui_filename( const BaseWindow *dialog )
 {
 	return( g_strdup( PKGDATADIR "/nact-assistant-export.ui" ));
 }
@@ -344,9 +347,12 @@ on_initial_load_dialog( NactAssistantExport *dialog, gpointer user_data )
 {
 	static const gchar *thisfn = "nact_assistant_export_on_initial_load_dialog";
 	GtkAssistant *assistant;
+	NactApplication *application;
+	NAUpdater *updater;
+	gboolean esc_quit, esc_confirm;
 
 	g_debug( "%s: dialog=%p, user_data=%p", thisfn, ( void * ) dialog, ( void * ) user_data );
-	g_assert( NACT_IS_ASSISTANT_EXPORT( dialog ));
+	g_return_if_fail( NACT_IS_ASSISTANT_EXPORT( dialog ));
 
 	assistant = GTK_ASSISTANT( base_window_get_toplevel( BASE_WINDOW( dialog )));
 
@@ -357,8 +363,12 @@ on_initial_load_dialog( NactAssistantExport *dialog, gpointer user_data )
 	assist_initial_load_confirm( dialog, assistant );
 	assist_initial_load_exportdone( dialog, assistant );
 
-	base_assistant_set_cancel_on_esc( BASE_ASSISTANT( dialog ), TRUE );
-	base_assistant_set_warn_on_esc( BASE_ASSISTANT( dialog ), TRUE );
+	application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( dialog )));
+	updater = nact_application_get_updater( application );
+	esc_quit = na_iprefs_read_bool( NA_IPREFS( updater ), IPREFS_ASSIST_ESC_QUIT, TRUE );
+	base_assistant_set_cancel_on_esc( BASE_ASSISTANT( dialog ), esc_quit );
+	esc_confirm = na_iprefs_read_bool( NA_IPREFS( updater ), IPREFS_ASSIST_ESC_CONFIRM, TRUE );
+	base_assistant_set_warn_on_esc( BASE_ASSISTANT( dialog ), esc_confirm );
 }
 
 static void
@@ -372,6 +382,12 @@ on_runtime_init_dialog( NactAssistantExport *dialog, gpointer user_data )
 
 	assistant = GTK_ASSISTANT( base_window_get_toplevel( BASE_WINDOW( dialog )));
 
+	base_window_signal_connect(
+			BASE_WINDOW( dialog ),
+			G_OBJECT( dialog ),
+			IACTIONS_LIST_SIGNAL_SELECTION_CHANGED,
+			G_CALLBACK( on_iactions_list_selection_changed ));
+
 	assist_runtime_init_intro( dialog, assistant );
 	assist_runtime_init_actions_list( dialog, assistant );
 	assist_runtime_init_target_folder( dialog, assistant );
@@ -380,6 +396,17 @@ on_runtime_init_dialog( NactAssistantExport *dialog, gpointer user_data )
 	assist_runtime_init_exportdone( dialog, assistant );
 }
 
+static void
+on_all_widgets_showed( NactAssistantExport *dialog )
+{
+	static const gchar *thisfn = "nact_assistant_export_on_all_widgets_showed";
+
+	g_debug( "%s: dialog=%p", thisfn, ( void * ) dialog );
+	g_return_if_fail( NACT_IS_ASSISTANT_EXPORT( dialog ));
+
+	nact_iactions_list_bis_select_first_row( NACT_IACTIONS_LIST( dialog ));
+}
+
 static void
 assist_initial_load_intro( NactAssistantExport *window, GtkAssistant *assistant )
 {
@@ -409,14 +436,14 @@ assist_initial_load_actions_list( NactAssistantExport *window, GtkAssistant *ass
 static void
 assist_runtime_init_actions_list( NactAssistantExport *window, GtkAssistant *assistant )
 {
-	GtkWidget *content;
-	NactApplication *appli;
-	NAPivot *pivot;
+	BaseWindow *parent;
 	GList *tree;
+	GtkWidget *content;
 
-	appli = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( window )));
-	pivot = nact_application_get_pivot( appli );
-	tree = na_pivot_get_items( pivot );
+	parent = base_window_get_parent( BASE_WINDOW( window ));
+	g_assert( NACT_IS_MAIN_WINDOW( parent ));
+	g_assert( NACT_IS_IACTIONS_LIST( parent ));
+	tree = nact_iactions_list_bis_get_items( NACT_IACTIONS_LIST( parent ));
 
 	nact_iactions_list_runtime_init_toplevel( NACT_IACTIONS_LIST( window ), tree );
 
@@ -428,8 +455,10 @@ assist_runtime_init_actions_list( NactAssistantExport *window, GtkAssistant *ass
 static void
 on_iactions_list_selection_changed( NactIActionsList *instance, GSList *selected_items )
 {
-	/*static const gchar *thisfn = "nact_assistant_export_on_actions_list_selection_changed";
-	g_debug( "%s: selection=%p, user_data=%p", thisfn, selection, user_data );*/
+	static const gchar *thisfn = "nact_assistant_export_on_actions_list_selection_changed";
+
+	g_debug( "%s: selection=%p, selected_items=%p (count=%d)",
+			thisfn, ( void * ) instance, ( void * ) selected_items, g_slist_length( selected_items ));
 
 	GtkAssistant *assistant;
 	gint pos;
@@ -474,7 +503,7 @@ assist_runtime_init_target_folder( NactAssistantExport *window, GtkAssistant *as
 {
 	GtkFileChooser *chooser;
 	NactApplication *application;
-	NAPivot *pivot;
+	NAUpdater *updater;
 	gchar *uri;
 	GtkWidget *content;
 
@@ -482,9 +511,9 @@ assist_runtime_init_target_folder( NactAssistantExport *window, GtkAssistant *as
 	gtk_file_chooser_unselect_all( chooser );
 
 	application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( window )));
-	pivot = nact_application_get_pivot( application );
+	updater = nact_application_get_updater( application );
 
-	uri = na_iprefs_read_string( NA_IPREFS( pivot ), IPREFS_EXPORT_ACTIONS_FOLDER_URI, "file:///tmp" );
+	uri = na_iprefs_read_string( NA_IPREFS( updater ), IPREFS_EXPORT_ITEMS_FOLDER_URI, "file:///tmp" );
 	if( uri && strlen( uri )){
 		gtk_file_chooser_set_uri( GTK_FILE_CHOOSER( chooser ), uri );
 	}
@@ -516,8 +545,6 @@ on_folder_selection_changed( GtkFileChooser *chooser, gpointer user_data )
 {
 	static const gchar *thisfn = "nact_assistant_export_on_folder_selection_changed";
 	GtkAssistant *assistant;
-	NactApplication *application;
-	NAPivot *pivot;
 	gint pos;
 	gchar *uri;
 	gboolean enabled;
@@ -534,14 +561,12 @@ on_folder_selection_changed( GtkFileChooser *chooser, gpointer user_data )
 
 		uri = gtk_file_chooser_get_uri( chooser );
 		g_debug( "%s: uri=%s", thisfn, uri );
-		enabled = ( uri && strlen( uri ) && na_utils_is_writable_dir( uri ));
+		enabled = ( uri && strlen( uri ) && na_core_utils_dir_is_writable_uri( uri ));
 
 		if( enabled ){
 			g_free( assist->private->uri );
 			assist->private->uri = g_strdup( uri );
-			application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( assist )));
-			pivot = nact_application_get_pivot( application );
-			na_iprefs_write_string( NA_IPREFS( pivot ), IPREFS_EXPORT_ACTIONS_FOLDER_URI, uri );
+			nact_iprefs_write_string( BASE_WINDOW( assist ), IPREFS_EXPORT_ITEMS_FOLDER_URI, uri );
 		}
 
 		g_free( uri );
@@ -555,92 +580,39 @@ on_folder_selection_changed( GtkFileChooser *chooser, gpointer user_data )
 static void
 assist_initial_load_format( NactAssistantExport *window, GtkAssistant *assistant )
 {
+	NactApplication *application;
+	NAUpdater *updater;
+	GtkWidget *container;
+
+	application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( window )));
+	updater = nact_application_get_updater( application );
+	container = base_window_get_widget( BASE_WINDOW( window ), "AssistantExportFormatVBox" );
+	nact_export_format_init_display( NA_PIVOT( updater ), container, EXPORT_FORMAT_DISPLAY_ASSISTANT );
 }
 
 static void
 assist_runtime_init_format( NactAssistantExport *window, GtkAssistant *assistant )
 {
-	GtkToggleButton *button;
 	GtkWidget *content;
-	NactApplication *application;
-	NAPivot *pivot;
-	gint format;
+	GtkWidget *container;
+	GQuark format;
 
-	application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( window )));
-	pivot = nact_application_get_pivot( application );
-	format = na_iprefs_get_export_format( NA_IPREFS( pivot ), IPREFS_EXPORT_FORMAT );
-
-	switch( format ){
-		case IPREFS_EXPORT_FORMAT_GCONF_SCHEMA_V1:
-			button = GTK_TOGGLE_BUTTON( get_gconfschemav1_button( window ));
-			gtk_toggle_button_set_active( button, TRUE );
-			break;
-
-		case IPREFS_EXPORT_FORMAT_GCONF_SCHEMA_V2:
-			button = GTK_TOGGLE_BUTTON( get_gconfschemav2_button( window ));
-			gtk_toggle_button_set_active( button, TRUE );
-			break;
-
-		case IPREFS_EXPORT_FORMAT_ASK:
-			button = GTK_TOGGLE_BUTTON( get_ask_button( window ));
-			gtk_toggle_button_set_active( button, TRUE );
-			break;
-
-		case IPREFS_EXPORT_FORMAT_GCONF_ENTRY:
-		default:
-			button = GTK_TOGGLE_BUTTON( get_gconfdump_button( window ));
-			gtk_toggle_button_set_active( button, TRUE );
-			break;
-	}
+	format = nact_iprefs_get_export_format( BASE_WINDOW( window ), IPREFS_EXPORT_FORMAT );
+	container = base_window_get_widget( BASE_WINDOW( window ), "AssistantExportFormatVBox" );
+	nact_export_format_select( container, format );
 
 	content = gtk_assistant_get_nth_page( assistant, ASSIST_PAGE_FORMAT_SELECTION );
 	gtk_assistant_set_page_complete( assistant, content, TRUE );
 }
 
-static GtkWidget *
-get_gconfschemav1_button( NactAssistantExport *window )
-{
-	return( base_window_get_widget( BASE_WINDOW( window ), "ExportSchemaV1Button" ));
-}
-
-static GtkWidget *
-get_gconfschemav2_button( NactAssistantExport *window )
-{
-	return( base_window_get_widget( BASE_WINDOW( window ), "ExportSchemaV2Button" ));
-}
-
-static GtkWidget *
-get_gconfdump_button( NactAssistantExport *window )
-{
-	return( base_window_get_widget( BASE_WINDOW( window ), "ExportGConfDumpButton" ));
-}
-
-static GtkWidget *
-get_ask_button( NactAssistantExport *window )
-{
-	return( base_window_get_widget( BASE_WINDOW( window ), "ExportAskButton" ));
-}
-
-static gint
+static NAExportFormat *
 get_export_format( NactAssistantExport *window )
 {
-	GtkToggleButton *gconf_schema_v1_button;
-	GtkToggleButton *gconf_schema_v2_button;
-	GtkToggleButton *ask_button;
-	gint format;
+	GtkWidget *container;
+	NAExportFormat *format;
 
-	gconf_schema_v1_button = GTK_TOGGLE_BUTTON( get_gconfschemav1_button( window ));
-	gconf_schema_v2_button = GTK_TOGGLE_BUTTON( get_gconfschemav2_button( window ));
-	ask_button = GTK_TOGGLE_BUTTON( get_ask_button( window ));
-
-	format = IPREFS_EXPORT_FORMAT_GCONF_ENTRY;
-	if( gtk_toggle_button_get_active( gconf_schema_v1_button )){
-		format = IPREFS_EXPORT_FORMAT_GCONF_SCHEMA_V1;
-	} else if( gtk_toggle_button_get_active( gconf_schema_v2_button )){
-		format = IPREFS_EXPORT_FORMAT_GCONF_SCHEMA_V2;
-	} else if( gtk_toggle_button_get_active( ask_button )){
-		format = IPREFS_EXPORT_FORMAT_ASK;
-	}
+	container = base_window_get_widget( BASE_WINDOW( window ), "AssistantExportFormatVBox" );
+	format = nact_export_format_get_selected( container );
 
 	return( format );
 }
@@ -691,84 +663,52 @@ static void
 assist_prepare_confirm( NactAssistantExport *window, GtkAssistant *assistant, GtkWidget *page )
 {
 	static const gchar *thisfn = "nact_assistant_export_prepare_confirm";
-	gchar *text, *tmp, *text2;
-	gchar *label1, *label2, *label3;
-	GList *actions, *ia;
-	NactApplication *application;
-	NAPivot *pivot;
-	gint format;
-
-	application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( window )));
-	pivot = nact_application_get_pivot( application );
+	GString *text;
+	gchar *label_item;
+	gchar *label11, *label12;
+	gchar *label21, *label22;
+	GList *items, *it;
+	NAExportFormat *format;
+	GtkLabel *confirm_label;
 
 	g_debug( "%s: window=%p, assistant=%p, page=%p",
 			thisfn, ( void * ) window, ( void * ) assistant, ( void * ) page );
 
 	/* i18n: this is the title of the confirm page of the export assistant */
-	text = g_strdup( _( "About to export selected actions:" ));
-	tmp = g_strdup_printf( "<b>%s</b>\n\n", text );
-	g_free( text );
-	text = tmp;
+	text = g_string_new( "" );
+	g_string_printf( text, "<b>%s</b>\n\n", _( "About to export selected items:" ));
 
-	actions = nact_iactions_list_get_selected_items( NACT_IACTIONS_LIST( window ));
-
-	for( ia = actions ; ia ; ia = ia->next ){
-		tmp = g_strdup_printf( "%s\t%s\n", text, na_object_get_label( ia->data ));
-		g_free( text );
-		text = tmp;
+	items = nact_iactions_list_bis_get_selected_items( NACT_IACTIONS_LIST( window ));
+	for( it = items ; it ; it = it->next ){
+		label_item = na_object_get_label( it->data );
+		g_string_append_printf( text, "\t%s\n", label_item );
+		g_free( label_item );
 	}
-
-	na_object_free_items_list( actions );
+	na_object_unref_selected_items( items );
 
 	g_assert( window->private->uri && strlen( window->private->uri ));
 
 	/* i18n: all exported actions go to one destination folder */
-	text2 = g_strdup( _( "Into the destination folder:" ));
-	tmp = g_strdup_printf( "%s\n\n<b>%s</b>\n\n\t%s", text, text2, window->private->uri );
-	g_free( text2 );
-	g_free( text );
-	text = tmp;
+	g_string_append_printf( text,
+			"\n\n<b>%s</b>\n\n\t%s", _( "Into the destination folder:" ), window->private->uri );
 
-	label1 = NULL;
-	label2 = NULL;
+	label11 = NULL;
+	label21 = NULL;
 	format = get_export_format( window );
-	switch( format ){
-		case IPREFS_EXPORT_FORMAT_GCONF_SCHEMA_V1:
-			label1 = g_strdup( gtk_button_get_label( GTK_BUTTON( get_gconfschemav1_button( window ))));
-			label2 = g_strdup( gtk_label_get_text( GTK_LABEL( base_window_get_widget( BASE_WINDOW( window ), "ExportSchemaV1Label"))));
-			break;
+	label11 = na_export_format_get_label( format );
+	label21 = na_export_format_get_description( format );
+	nact_iprefs_set_export_format( BASE_WINDOW( window ), IPREFS_EXPORT_FORMAT, na_export_format_get_quark( format ));
+	label12 = na_core_utils_str_remove_char( label11, "_" );
+	label22 = na_core_utils_str_add_prefix( "\t", label21 );
+	g_string_append_printf( text, "\n\n<b>%s</b>\n\n%s", label12, label22 );
+	g_free( label22 );
+	g_free( label21 );
+	g_free( label12 );
+	g_free( label11 );
 
-		case IPREFS_EXPORT_FORMAT_GCONF_SCHEMA_V2:
-			label1 = g_strdup( gtk_button_get_label( GTK_BUTTON( get_gconfschemav2_button( window ))));
-			label2 = g_strdup( gtk_label_get_text( GTK_LABEL( base_window_get_widget( BASE_WINDOW( window ), "ExportSchemaV2Label"))));
-			break;
-
-		case IPREFS_EXPORT_FORMAT_GCONF_ENTRY:
-			label1 = g_strdup( gtk_button_get_label( GTK_BUTTON( get_gconfdump_button( window ))));
-			label2 = g_strdup( gtk_label_get_text( GTK_LABEL( base_window_get_widget( BASE_WINDOW( window ), "ExportGConfDumpLabel"))));
-			break;
-
-		case IPREFS_EXPORT_FORMAT_ASK:
-			label1 = g_strdup( gtk_button_get_label( GTK_BUTTON( get_ask_button( window ))));
-			label2 = g_strdup( gtk_label_get_text( GTK_LABEL( base_window_get_widget( BASE_WINDOW( window ), "ExportAskLabel"))));
-			break;
-
-		default:
-			g_return_if_reached();
-			break;
-	}
-	na_iprefs_set_export_format( NA_IPREFS( pivot ), IPREFS_EXPORT_FORMAT, format );
-
-	label3 = na_utils_prefix_strings( "\t", label2 );
-	tmp = g_strdup_printf( "%s\n\n<b>%s</b>\n\n%s", text, label1, label3 );
-	g_free( label3 );
-	g_free( label2 );
-	g_free( label1 );
-	g_free( text );
-	text = tmp;
-
-	gtk_label_set_markup( GTK_LABEL( page ), text );
-	g_free( text );
+	confirm_label = GTK_LABEL( base_window_get_widget( BASE_WINDOW( window ), "AssistantExportConfirmLabel" ));
+	gtk_label_set_markup( confirm_label, text->str );
+	g_string_free( text, TRUE );
 
 	gtk_assistant_set_page_complete( assistant, page, TRUE );
 }
@@ -783,19 +723,18 @@ assistant_apply( BaseAssistant *wnd, GtkAssistant *assistant )
 {
 	static const gchar *thisfn = "nact_assistant_export_on_apply";
 	NactAssistantExport *window;
-	NactApplication *application;
-	NAPivot *pivot;
 	GList *actions, *ia;
 	ExportStruct *str;
+	NactApplication *application;
+	NAUpdater *updater;
 
 	g_debug( "%s: window=%p, assistant=%p", thisfn, ( void * ) wnd, ( void * ) assistant );
 	g_assert( NACT_IS_ASSISTANT_EXPORT( wnd ));
 	window = NACT_ASSISTANT_EXPORT( wnd );
 
 	application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( window )));
-	pivot = nact_application_get_pivot( application );
-
-	actions = nact_iactions_list_get_selected_items( NACT_IACTIONS_LIST( window ));
+	updater = nact_application_get_updater( application );
+	actions = nact_iactions_list_bis_get_selected_items( NACT_IACTIONS_LIST( window ));
 
 	g_assert( window->private->uri && strlen( window->private->uri ));
 
@@ -803,22 +742,24 @@ assistant_apply( BaseAssistant *wnd, GtkAssistant *assistant )
 		str = g_new0( ExportStruct, 1 );
 		window->private->results = g_list_append( window->private->results, str );
 
-		str->action = NA_OBJECT_ACTION( na_iduplicable_get_origin( NA_IDUPLICABLE( ia->data )));
+		str->item = NA_OBJECT_ITEM( na_object_get_origin( NA_IDUPLICABLE( ia->data )));
+
+		str->format = nact_iprefs_get_export_format( BASE_WINDOW( wnd ), IPREFS_EXPORT_FORMAT );
 
-		str->format = na_iprefs_get_export_format( NA_IPREFS( pivot ), IPREFS_EXPORT_FORMAT );
 		if( str->format == IPREFS_EXPORT_FORMAT_ASK ){
-			str->format = nact_assistant_export_ask_user( BASE_WINDOW( wnd ), str->action );
+			str->format = nact_export_ask_user( BASE_WINDOW( wnd ), str->item );
+
 			if( str->format == IPREFS_EXPORT_NO_EXPORT ){
 				str->msg = g_slist_append( NULL, g_strdup( _( "Export canceled due to user action." )));
 			}
 		}
 
 		if( str->format != IPREFS_EXPORT_NO_EXPORT ){
-			str->fname = na_xml_writer_export( str->action, window->private->uri, str->format, &str->msg );
+			str->fname = na_exporter_to_file( NA_PIVOT( updater ), str->item, window->private->uri, str->format, &str->msg );
 		}
 	}
 
-	na_object_free_items_list( actions );
+	na_object_unref_selected_items( actions );
 }
 
 static void
@@ -831,13 +772,19 @@ assist_prepare_exportdone( NactAssistantExport *window, GtkAssistant *assistant,
 	gchar *label;
 	GSList *is;
 	gint errors;
+	GtkTextView *summary_textview;
+	GtkTextBuffer *summary_buffer;
+	GtkTextTag *title_tag;
+	GtkTextIter start, end;
+	gint title_len;
 
 	g_debug( "%s: window=%p, assistant=%p, page=%p",
 			thisfn, ( void * ) window, ( void * ) assistant, ( void * ) page );
 
 	/* i18n: result of the export assistant */
 	text = g_strdup( _( "Selected actions have been proceeded :" ));
-	tmp = g_strdup_printf( "<b>%s</b>\n\n", text );
+	title_len = g_utf8_strlen( text, -1 );
+	tmp = g_strdup_printf( "%s\n\n", text );
 	g_free( text );
 	text = tmp;
 
@@ -846,7 +793,7 @@ assist_prepare_exportdone( NactAssistantExport *window, GtkAssistant *assistant,
 	for( ir = window->private->results ; ir ; ir = ir->next ){
 		str = ( ExportStruct * ) ir->data;
 
-		label = na_object_get_label( str->action );
+		label = na_object_get_label( str->item );
 		tmp = g_strdup_printf( "%s\t%s\n", text, label );
 		g_free( text );
 		text = tmp;
@@ -882,9 +829,21 @@ assist_prepare_exportdone( NactAssistantExport *window, GtkAssistant *assistant,
 		text = tmp;
 	}
 
-	gtk_label_set_markup( GTK_LABEL( page ), text );
+	summary_textview = GTK_TEXT_VIEW( base_window_get_widget( BASE_WINDOW( window ), "AssistantExportSummaryTextView" ));
+	summary_buffer = gtk_text_view_get_buffer( summary_textview );
+	gtk_text_buffer_set_text( summary_buffer, text, -1 );
 	g_free( text );
 
+	title_tag = gtk_text_buffer_create_tag( summary_buffer, "title",
+			"weight", PANGO_WEIGHT_BOLD,
+			NULL );
+
+	gtk_text_buffer_get_iter_at_offset( summary_buffer, &start, 0 );
+	gtk_text_buffer_get_iter_at_offset( summary_buffer, &end, title_len );
+	gtk_text_buffer_apply_tag( summary_buffer, title_tag, &start, &end );
+
+	g_object_unref( title_tag );
+
 	gtk_assistant_set_page_complete( assistant, page, TRUE );
 	base_assistant_set_warn_on_cancel( BASE_ASSISTANT( window ), FALSE );
 	base_assistant_set_warn_on_esc( BASE_ASSISTANT( window ), FALSE );
@@ -899,7 +858,7 @@ free_results( GList *list )
 	for( ir = list ; ir ; ir = ir->next ){
 		str = ( ExportStruct * ) ir->data;
 		g_free( str->fname );
-		na_utils_free_string_list( str->msg );
+		na_core_utils_slist_free( str->msg );
 	}
 
 	g_list_free( list );
